@@ -6,6 +6,7 @@
 #include "abstractmetafield.h"
 #include "abstractmetafunction.h"
 #include "abstractmetalang.h"
+#include "include.h"
 #include "modifications.h"
 #include "sourcelocation.h"
 #include "typedatabase.h"
@@ -16,11 +17,14 @@
 
 #include "qtcompat.h"
 
-#include <QtCore/QDebug>
-#include <QtCore/QDir>
-#include <QtCore/QFile>
-#include <QtCore/QStringList>
-#include <QtCore/QXmlStreamReader>
+#include <QtCore/qdebug.h>
+#include <QtCore/qdir.h>
+#include <QtCore/qfile.h>
+#include <QtCore/qstringlist.h>
+#include <QtCore/qxmlstream.h>
+
+#include <algorithm>
+#include <iterator>
 
 using namespace Qt::StringLiterals;
 
@@ -42,6 +46,20 @@ static QTextStream &operator<<(QTextStream &s, Access a)
     return s;
 }
 
+QString msgModificationCandidates(const AbstractMetaFunctionCPtr &function)
+{
+    QString result;
+    const auto &signatures = function->modificationSignatures();
+    for (qsizetype i = 0, size = signatures.size(); i < size; ++i) {
+        if (i > 0)
+            result += " / "_L1;
+        result += u'"' + signatures.at(i) + u'"';
+    }
+    if (auto klazz = function->implementingClass())
+        result += " in "_L1 + klazz->name();
+    return result;
+}
+
 QString msgNoFunctionForModification(const AbstractMetaClassCPtr &klass,
                                      const QString &signature,
                                      const QString &originalSignature,
@@ -71,6 +89,19 @@ QString msgNoFunctionForModification(const AbstractMetaClassCPtr &klass,
     return result;
 }
 
+QString msgModificationConstMismatch(const AbstractMetaFunctionCPtr &function,
+                                     const QString &modificationSignature)
+{
+    QString result;
+    QTextStream str(&result);
+    if (auto klazz = function->implementingClass())
+        str << klazz->typeEntry()->sourceLocation();
+    str << "signature \"" << modificationSignature
+        << "\" needs \"const\" to fully match \"" << function->classQualifiedSignature()
+        << "\". The partial matching will be removed in a future release.";
+    return result;
+}
+
 QString msgArgumentIndexOutOfRange(const AbstractMetaFunction *func, int index)
 {
     QString result;
@@ -408,6 +439,16 @@ QString msgStrippingArgument(const FunctionModelItem &f, int i,
     return result;
 }
 
+QString msgStrippingQtDisambiguatedArgument(const FunctionModelItem &f,
+                                            const QString &originalSignature)
+{
+    QString result;
+    QTextStream str(&result);
+    str << f->sourceLocation() << "Stripping last argument of "
+        << originalSignature << " (QT6_DECL_NEW_OVERLOAD_TAIL).";
+    return result;
+}
+
 QString msgEnumNotDefined(const EnumTypeEntryCPtr &t)
 {
     QString result;
@@ -481,6 +522,21 @@ QString msgCannotFindTypeEntryForSmartPointer(const QString &t, const QString &s
         + u"\" for instantiation of \""_s +smartPointerType + u"\"."_s;
 }
 
+QString msgInheritTemplateIssue(const AbstractMetaClassPtr &subclass,
+                                const TypeInfo &info,
+                                const QString &what)
+{
+    return "While inheriting template "_L1 + subclass->name()
+           + " from "_L1 + info.toString() + ": "_L1 + what;
+}
+
+QString msgIgnoringTemplateParameter(const QString &typeName,
+                                     const char *why)
+{
+    return "Ignoring template parameter "_L1 + typeName +
+           ": "_L1 + QLatin1StringView(why);
+}
+
 QString msgInvalidSmartPointerType(const TypeInfo &i)
 {
     return u"Invalid smart pointer type \""_s +i.toString() + u"\"."_s;
@@ -874,15 +930,6 @@ QString msgCannotFindView(const QString &viewedName, const QString &name)
         + u" for "_s + name;
 }
 
-QString msgCannotFindSnippet(const QString &file, const QString &snippetLabel)
-{
-    QString result;
-    QTextStream str(&result);
-    str << "Cannot find snippet \"" << snippetLabel << "\" in "
-        << QDir::toNativeSeparators(file) << '.';
-    return result;
-}
-
 QString msgSnippetError(const QString &context, const char *what)
 {
     return "Error processing code snippet of "_L1 + context
@@ -980,3 +1027,88 @@ QString msgInvalidLanguageLevel(const QString &l)
 {
     return u"Invalid argument for language level: \""_s + l + u"\"."_s;
 }
+
+QString msgCannotFindImage(const QString &href, const QString &context,
+                           const QStringList &candidates)
+{
+    QString result = "Cannot resolve image "_L1 + href + " for "_L1 + context
+                     + " (tried "_L1;
+    for (const auto &candidate : candidates)
+        result += QDir::toNativeSeparators(candidate) + u' ';
+    result += ")."_L1;
+    return result;
+}
+
+QString msgNotRelative(const QString &path, const QString &dir)
+{
+    return QDir::toNativeSeparators(path)
+           + " is not under "_L1 + QDir::toNativeSeparators(dir) + u'.';
+}
+
+QString msgCannotCreateDir(const QString &dir)
+{
+    return "Cannot create "_L1 + QDir::toNativeSeparators(dir) + u'.';
+}
+
+QString msgCannotCopy(const QFile &source, const QString &target)
+{
+    return "Cannot copy "_L1 + QDir::toNativeSeparators(source.fileName())
+        + " to "_L1 + QDir::toNativeSeparators(target)
+        + ": "_L1 + source.errorString();
+}
+
+QString msgCannotFindQDocFile(const AbstractMetaClassCPtr &metaClass,
+                              const QStringList &candidates)
+{
+    QStringList nativeCandidates;
+    std::transform(candidates.cbegin(), candidates.cend(), std::back_inserter(nativeCandidates),
+                   QDir::toNativeSeparators);
+    QString result;
+    QTextStream(&result) << "Cannot find qdoc file for "
+        << (metaClass->isNamespace() ? "namespace" : "class") << " \""
+        << metaClass->typeEntry()->qualifiedCppName() << "\" ("
+        << QDir::toNativeSeparators(metaClass->typeEntry()->include().name())
+        << "), tried: " << nativeCandidates.join(", "_L1);
+    return result;
+}
+
+QString msgCannotCall(const AbstractMetaFunctionCPtr &func,
+                      int arg, bool injectCodeCallsFunc, bool hasConversionRule)
+{
+    QString result;
+    QTextStream str(&result);
+    str << "No way to generate a binding call for \"" << func->ownerClass()->name() << "::"
+        << func->signature() << '"';
+    if (func->isUserAdded())
+        str << " (user added)";
+    str << " with the modifications for argument " << (arg + 1) << ':';
+    if (!injectCodeCallsFunc)
+        str << " There is no code injection calling the function.";
+    if (!hasConversionRule)
+        str << " There is no conversion rule.";
+    return result;
+}
+
+QString msgRemoveRedundantOverload(const AbstractMetaFunctionCPtr &func,
+                                   const QString &type)
+{
+    return "Removing \""_L1 + func->classQualifiedSignature()
+           + "\" due to presence of an overload taking a \""_L1
+           + type + "\" parameter."_L1;
+}
+
+QString msgCommandLineArguments(const QStringList &argv)
+{
+    QString result = "Command line:\n   "_L1;
+    for (const QString &arg : argv) {
+        result.append(u' ');
+        const bool quote = arg.contains(u' ');
+        if (quote)
+            result.append(u'"');
+        result.append(arg);
+        if (quote)
+            result.append(u'"');
+    }
+    result.append(u'\n');
+    return result;
+}
